'''
    An example script to cluster reconstructed calorimeter hits
'''

from Gaudi.Configuration import *
import os
import ROOT

from GaudiKernel.DataObjectHandleBase import DataObjectHandleBase
from Configurables import ApplicationMgr, EICDataSvc, PodioInput, PodioOutput, GeoSvc
from GaudiKernel.SystemOfUnits import MeV, GeV, mm, cm, mrad

detector_name = str(os.environ.get("JUGGLER_DETECTOR", "athena"))
detector_path = str(os.environ.get("JUGGLER_DETECTOR_PATH", "."))
compact_path = os.path.join(detector_path, detector_name)

# get sampling fractions from system environment variable, 1.0 by default
ce_ecal_sf = float(os.environ.get("CE_ECAL_SAMP_FRAC", 0.253))
cb_ecal_sf = float(os.environ.get("CB_ECAL_SAMP_FRAC", 0.01324))
cb_hcal_sf = float(os.environ.get("CB_HCAL_SAMP_FRAC", 0.038))
ci_hcal_sf = float(os.environ.get("CI_HCAL_SAMP_FRAC", 0.025))
ce_hcal_sf = float(os.environ.get("CE_HCAL_SAMP_FRAC", 0.025))

# input and output
input_digis = [f.strip() for f in str.split(os.environ["JUGGLER_DIGI_FILE"], ",") if f.strip()]
output_rec = str(os.environ["JUGGLER_REC_FILE"])
n_events = int(os.environ["JUGGLER_N_EVENTS"])

# geometry service
geo_service = GeoSvc("GeoSvc", detectors=["{}.xml".format(compact_path)], OutputLevel=INFO)
# data service
podioevent = EICDataSvc("EventDataSvc", inputs=input_digis)


# juggler components
from Configurables import Jug__Base__InputCopier_dd4pod__Geant4ParticleCollection_dd4pod__Geant4ParticleCollection_ as MCCopier
from Configurables import Jug__Digi__CalorimeterHitDigi as CalHitDigi
from Configurables import Jug__Reco__CalorimeterHitReco as CalHitReco
from Configurables import Jug__Reco__CalorimeterHitsMerger as CalHitsMerger
from Configurables import Jug__Reco__CalorimeterIslandCluster as IslandCluster

from Configurables import Jug__Reco__ImagingPixelReco as ImCalPixelReco
from Configurables import Jug__Reco__ImagingTopoCluster as ImagingCluster

from Configurables import Jug__Reco__ClusterRecoCoG as RecoCoG
from Configurables import Jug__Reco__ImagingClusterReco as ImagingClusterReco

# branches needed from simulation root file
digi_coll = [
#    "mcparticles2",
    "EcalEndcapNHitsReco",
    "EcalEndcapPHitsReco",
    "EcalBarrelHitsReco",
    "HcalBarrelHitsReco",
    "HcalHadronEndcapHitsReco",
    "HcalElectronEndcapHitsReco",
]

# input and output
podin = PodioInput("PodioReader", collections=digi_coll, OutputLevel=DEBUG)
podout = PodioOutput("out", filename=output_rec)
# copier needed to get around input --> output copy bug. So truth (mcparticles) can be saved in output file
# copier = MCCopier("MCCopier", inputCollection="mcparticles2", outputCollection="mcparticles3")


# Crystal Endcap Ecal
ce_ecal_cl = IslandCluster("ce_ecal_cl",
        # OutputLevel=DEBUG,
        inputHitCollection="EcalEndcapNHitsReco",
        outputClusterCollection="EcalEndcapNProtoClusters",
        splitHitCollection="EcalEndcapNHitsSplit",
        splitCluster=False,
        minClusterCenterEdep=30*MeV,
        groupRanges=[2.2*cm, 2.2*cm])

ce_ecal_clreco = RecoCoG("ce_ecal_clreco",
        inputClusterCollection="EcalEndcapNProtoClusters", 
        outputClusterCollection="EcalEndcapNClusters", 
        samplingFraction=0.998,      # this accounts for a small fraction of leakage
        logWeightBase=4.6)


# Endcap Sampling Ecal
# merge hits in different layer (projection to local x-y plane)
ci_ecal_merger = CalHitsMerger("ci_ecal_merger",
        inputHitCollection="EcalEndcapPHitsReco",
        outputHitCollection="EcalEndcapPHitsRecoXY",
        fields=["layer", "slice"],
        fieldRefNumbers=[1, 0],
        readoutClass="EcalEndcapPHits")

ci_ecal_cl = IslandCluster("ci_ecal_cl",
        inputHitCollection="EcalEndcapPHitsRecoXY",
        outputClusterCollection="EcalEndcapPProtoClusters",
        splitHitCollection="EcalEndcapPHitsSplit",
        splitCluster=False,
        minClusterCenterEdep=30.*MeV,
        groupRanges=[5*mm, 5*mm])

ci_ecal_clreco = RecoCoG("ci_ecal_clreco",
        inputClusterCollection="EcalEndcapPProtoClusters", 
        outputClusterCollection="EcalEndcapPClusters", 
        logWeightBase=6.2,
        samplingFraction=ce_ecal_sf)


# Central Barrel Ecal (Imaging Cal.)
cb_ecal_cl = ImagingCluster("cb_ecal_cl",
        inputHitCollection="EcalBarrelHitsReco",
        outputClusterCollection="EcalBarrelProtoClusters",
        outputHitCollection="EcalBarrelClusterHits",
        localRanges=[2.*mm, 2*mm],              # same layer
        adjLayerRanges=[10*mrad, 10*mrad],      # adjacent layer
        adjLayerDiff=2,                         # id diff for adjacent layer
        adjSectorDist=3.*cm)                    # different sector
cb_ecal_clreco = ImagingClusterReco("cb_ecal_clreco",
        samplingFraction=cb_ecal_sf,
        inputClusterCollection="EcalBarrelProtoClusters",
        outputClusterCollection="EcalBarrelClusters",
        outputLayerCollection="EcalBarrelLayers")


# Central Barrel Hcal
# merge hits in different layer (projection to local x-y plane)
cb_hcal_merger = CalHitsMerger("cb_hcal_merger",
        inputHitCollection="HcalBarrelHitsReco",
        outputHitCollection="HcalBarrelHitsRecoXY",
        readoutClass="HcalBarrelHits",
        fields=["layer", "slice"],
        fieldRefNumbers=[1, 0])

cb_hcal_cl = IslandCluster("cb_hcal_cl",
        inputHitCollection="HcalBarrelHitsRecoXY",
        outputClusterCollection="HcalBarrelProtoClusters",
        splitHitCollection="HcalBarrelHitsSplit",
        splitCluster=False,
        minClusterCenterEdep=30.*MeV,
        groupRanges=[15.*cm, 15.*cm])

cb_hcal_clreco = RecoCoG("cb_hcal_clreco",
        inputClusterCollection="HcalBarrelProtoClusters", 
        outputClusterCollection="HcalBarrelClusters", 
        logWeightBase=6.2,
        samplingFraction=cb_hcal_sf)


# Hcal Hadron Endcap HCal
ci_hcal_merger = CalHitsMerger("ci_hcal_merger",
        inputHitCollection="HcalHadronEndcapHitsReco",
        outputHitCollection="HcalHadronEndcapHitsRecoXY",
        readoutClass="HcalHadronEndcapHits",
        fields=["layer", "slice"],
        fieldRefNumbers=[1, 0])

ci_hcal_cl = IslandCluster("ci_hcal_cl",
        inputHitCollection="HcalHadronEndcapHitsRecoXY",
        outputClusterCollection="HcalHadronEndcapProtoClusters",
        splitHitCollection="HcalHadronEndcapHitsSplit",
        splitCluster=False,
        minClusterCenterEdep=30.*MeV,
        groupRanges=[15.*cm, 15.*cm])

ci_hcal_clreco = RecoCoG("ci_hcal_clreco",
        inputClusterCollection="HcalHadronEndcapProtoClusters", 
        outputClusterCollection="HcalHadronEndcapClusters", 
        logWeightBase=6.2,
        samplingFraction=ci_hcal_sf)

# Hcal Electron Endcap
ce_hcal_merger = CalHitsMerger("ce_hcal_merger",
        inputHitCollection="HcalElectronEndcapHitsReco",
        outputHitCollection="HcalElectronEndcapHitsRecoXY",
        readoutClass="HcalElectronEndcapHits",
        fields=["layer", "slice"],
        fieldRefNumbers=[1, 0])

ce_hcal_cl = IslandCluster("ce_hcal_cl",
        inputHitCollection="HcalElectronEndcapHitsRecoXY",
        outputClusterCollection="HcalElectronEndcapProtoClusters",
        splitHitCollection="HcalElectronEndcapHitsSplit",
        splitCluster=False,
        minClusterCenterEdep=30.*MeV,
        groupRanges=[15.*cm, 15.*cm])

ce_hcal_clreco = RecoCoG("ce_hcal_clreco",
        inputClusterCollection="HcalElectronEndcapProtoClusters", 
        outputClusterCollection="HcalElectronEndcapClusters", 
        logWeightBase=6.2,
        samplingFraction=ce_hcal_sf)


podout.outputCommands = ["drop *", "keep mcparticles", "keep *Clusters", "keep *Layers"]

ApplicationMgr(
    TopAlg = [podin, # copier,
              ce_ecal_cl, ce_ecal_clreco,
              ci_ecal_merger, ci_ecal_cl, ci_ecal_clreco,
              cb_ecal_cl, cb_ecal_clreco,
              cb_hcal_merger, cb_hcal_cl, cb_hcal_clreco,
              ce_hcal_merger, ce_hcal_cl, ce_hcal_clreco,
              ci_hcal_merger, ci_hcal_cl, ci_hcal_clreco,
              podout],
    EvtSel = 'NONE',
    EvtMax   = n_events,
    ExtSvc = [podioevent],
    OutputLevel=DEBUG
)

