'''
    An example script to digitize/reconstruct calorimeter hits
'''
from Gaudi.Configuration import *
import os
import ROOT

from GaudiKernel.DataObjectHandleBase import DataObjectHandleBase
from Configurables import ApplicationMgr, EICDataSvc, PodioInput, PodioOutput, GeoSvc
from GaudiKernel.SystemOfUnits import MeV, GeV, mm, cm, mrad

detector_name = str(os.environ.get("JUGGLER_DETECTOR", "athena"))
detector_path = str(os.environ.get("JUGGLER_DETECTOR_PATH", "."))
compact_path = os.path.join(detector_path, detector_name)

# input and output
input_sims = [f.strip() for f in str.split(os.environ["JUGGLER_SIM_FILE"], ",") if f.strip()]
output_rec = str(os.environ["JUGGLER_DIGI_FILE"])
n_events = int(os.environ["JUGGLER_N_EVENTS"])

# geometry service
geo_service = GeoSvc("GeoSvc", detectors=["{}.xml".format(compact_path)], OutputLevel=INFO)
# data service
podioevent = EICDataSvc("EventDataSvc", inputs=input_sims)


# juggler components
from Configurables import Jug__Base__InputCopier_dd4pod__Geant4ParticleCollection_dd4pod__Geant4ParticleCollection_ as MCCopier
from Configurables import Jug__Digi__CalorimeterHitDigi as CalHitDigi
from Configurables import Jug__Reco__CalorimeterHitReco as CalHitReco
from Configurables import Jug__Reco__CalorimeterHitsMerger as CalHitsMerger
from Configurables import Jug__Reco__CalorimeterIslandCluster as IslandCluster

from Configurables import Jug__Reco__ImagingPixelReco as ImCalPixelReco
from Configurables import Jug__Reco__ImagingTopoCluster as ImagingCluster

from Configurables import Jug__Reco__ClusterRecoCoG as RecoCoG
from Configurables import Jug__Reco__ImagingClusterReco as ImagingClusterReco

# branches needed from simulation root file
sim_coll = [
    "mcparticles",
    "EcalEndcapNHits",
    "EcalEndcapPHits",
    "EcalBarrelHits",
    "HcalBarrelHits",
    "HcalHadronEndcapHits",
    "HcalElectronEndcapHits",
]

# input and output
podin = PodioInput("PodioReader", collections=sim_coll)
podout = PodioOutput("out", filename=output_rec)
# copier needed to get around input --> output copy bug. So truth (mcparticles) can be saved in output file
copier = MCCopier("MCCopier", inputCollection="mcparticles", outputCollection="mcparticles2")


# Crystal Endcap Ecal
ce_ecal_digi = CalHitDigi("ce_ecal_digi",
        inputHitCollection="EcalEndcapNHits",
        outputHitCollection="EcalEndcapNHitsDigi",
        energyResolutions=[0., 0.02, 0.],
        dynamicRangeADC=5.*GeV,     # digi settings must match with reco
        capacityADC=32768,
        pedestalMean=400,
        pedestalSigma=3)

ce_ecal_reco = CalHitReco("ce_ecal_reco",
        inputHitCollection="EcalEndcapNHitsDigi",
        outputHitCollection="EcalEndcapNHitsReco",
        dynamicRangeADC=5.*GeV,
        capacityADC=32768,
        pedestalMean=400,
        pedestalSigma=3,
        thresholdFactor=4,          # 4 sigma
        minimumHitEdep=1.0*MeV,     # discard low energy hits
        readoutClass="EcalEndcapNHits",
        sectorField="sector")

# Endcap Sampling Ecal
ci_ecal_digi = CalHitDigi("ci_ecal_digi",
         inputHitCollection="EcalEndcapPHits",
         outputHitCollection="EcalEndcapPHitsDigi",
         dynamicRangeADC=50.*MeV,
         capacityADC=32768,
         pedestalMean=400,
         pedestalSigma=10)

ci_ecal_reco = CalHitReco("ci_ecal_reco",
        inputHitCollection="EcalEndcapPHitsDigi",
        outputHitCollection="EcalEndcapPHitsReco",
        dynamicRangeADC=50.*MeV,
        capacityADC=32768,
        pedestalMean=400,
        pedestalSigma=10,
        thresholdFactor=5.0)

# Central Barrel Ecal (Imaging Cal.)
cb_ecal_digi = CalHitDigi("cb_ecal_digi",
        inputHitCollection="EcalBarrelHits",
        outputHitCollection="EcalBarrelHitsDigi",
        energyResolutions=[0., 0.02, 0.],   # 2% flat resolution
        dynamicRangeADC=3*MeV,
        capacityADC=8192,
        pedestalMean=400,
        pedestalSigma=20)   # about 6 keV
cb_ecal_reco = ImCalPixelReco("cb_ecal_reco",
        inputHitCollection="EcalBarrelHitsDigi",
        outputHitCollection="EcalBarrelHitsReco",
        dynamicRangeADC=3.*MeV,
        capacityADC=8192,
        pedestalMean=400,
        pedestalSigma=20,
        thresholdFactor=3,  # about 20 keV
        readoutClass="EcalBarrelHits",  # readout class
        layerField="layer",             # field to get layer id
        sectorField="module")           # field to get sector id

# Central Barrel Hcal
cb_hcal_digi = CalHitDigi("cb_hcal_digi",
         inputHitCollection="HcalBarrelHits",
         outputHitCollection="HcalBarrelHitsDigi",
         dynamicRangeADC=50.*MeV,
         capacityADC=32768,
         pedestalMean=400,
         pedestalSigma=10)

cb_hcal_reco = CalHitReco("cb_hcal_reco",
        inputHitCollection="HcalBarrelHitsDigi",
        outputHitCollection="HcalBarrelHitsReco",
        dynamicRangeADC=50.*MeV,
        capacityADC=32768,
        pedestalMean=400,
        pedestalSigma=10,
        thresholdFactor=5.0,
        readoutClass="HcalBarrelHits",
        layerField="layer",
        sectorField="module")

# Hcal Hadron Endcap
ci_hcal_digi = CalHitDigi("ci_hcal_digi",
         inputHitCollection="HcalHadronEndcapHits",
         outputHitCollection="HcalHadronEndcapHitsDigi",
         dynamicRangeADC=50.*MeV,
         capacityADC=32768,
         pedestalMean=400,
         pedestalSigma=10)

ci_hcal_reco = CalHitReco("ci_hcal_reco",
        inputHitCollection="HcalHadronEndcapHitsDigi",
        outputHitCollection="HcalHadronEndcapHitsReco",
        dynamicRangeADC=50.*MeV,
        capacityADC=32768,
        pedestalMean=400,
        pedestalSigma=10,
        thresholdFactor=5.0)

# Hcal Electron Endcap
ce_hcal_digi = CalHitDigi("ce_hcal_digi",
        inputHitCollection="HcalElectronEndcapHits",
        outputHitCollection="HcalElectronEndcapHitsDigi",
        dynamicRangeADC=50.*MeV,
        capacityADC=32768,
        pedestalMean=400,
        pedestalSigma=10)

ce_hcal_reco = CalHitReco("ce_hcal_reco",
        inputHitCollection="HcalElectronEndcapHitsDigi",
        outputHitCollection="HcalElectronEndcapHitsReco",
        dynamicRangeADC=50.*MeV,
        capacityADC=32768,
        pedestalMean=400,
        pedestalSigma=10,
        thresholdFactor=5.0)


podout.outputCommands = ['drop *', 'keep mcparticles2', 'keep *Reco', 'keep *Digi']

ApplicationMgr(
    TopAlg = [podin, copier,
              ce_ecal_digi, ce_ecal_reco,
              ci_ecal_digi, ci_ecal_reco,
              cb_ecal_digi, cb_ecal_reco,
              cb_hcal_digi, cb_hcal_reco,
              ce_hcal_digi, ce_hcal_reco,
              ci_hcal_digi, ci_hcal_reco,
              podout],
    EvtSel = 'NONE',
    EvtMax   = n_events,
    ExtSvc = [podioevent],
    OutputLevel=DEBUG
)

