'''
    A simple analysis script to extract some basic info of Monte-Carlo particles and clusters
'''
import os
import ROOT
import pandas as pd
import numpy as np
import argparse
from matplotlib import pyplot as plt
import matplotlib.ticker as ticker


# load root macros, input is an argument string
def load_root_macros(arg_macros):
    for path in arg_macros.split(','):
        path = path.strip()
        if os.path.exists(path):
            gROOT.Macro(path)
            print('Loading root macro: \'{}\' loaded.'.format(path))
        else:
            print('Loading root macros: \'{}\' does not exist, skip loading it.'.format(path))


# read from RDataFrame and flatten a given collection, return pandas dataframe
def flatten_collection(rdf, collection, cols=None):
    if not cols:
        cols = [str(c) for c in rdf.GetColumnNames() if str(c).startswith('{}.'.format(collection))]
    else:
        cols = ['{}.{}'.format(collection, c) for c in cols]
    if not cols:
        print('cannot find any branch under collection {}'.format(collection))
        return pd.DataFrame()

    data = rdf.AsNumpy(cols)
    # flatten the data, add an event id to identify clusters from different events
    evns = []
    for i, vec in enumerate(data[cols[0]]):
        evns += [i]*vec.size()
    for n, vals in data.items():
        # make sure ints are not converted to floats
        typename = vals[0].__class__.__name__.lower()
        dtype = np.int64 if 'int' in typename or 'long' in typename else np.float64
        # type safe creation
        data[n] = np.asarray([v for vec in vals for v in vec], dtype=dtype)
    # build data frame
    dfp = pd.DataFrame({c: pd.Series(v) for c, v in data.items()})
    dfp.loc[:, 'event'] = evns
    return dfp


def thrown_particles_figure(rdf, save, mcbranch="mcparticles"):
    # define truth particle info
    df = flatten_collection(rdf, mcbranch, ['genStatus', 'pdgID', 'psx', 'psy', 'psz', 'mass'])
    df.rename(columns={c: c.replace(mcbranch + '.', '') for c in df.columns}, inplace=True)
    # select thrown particles
    df = df[df['genStatus'] == 1]

    # figure
    fig, axs = plt.subplots(2, 2, figsize=(16, 12), dpi=120)
    # convert pid to particle names
    pdgbase = ROOT.TDatabasePDG()
    get_pname = np.vectorize(lambda pid: pdgbase.GetParticle(int(pid)).GetName())

    # enumerate particle names
    df.loc[:, 'pname'] = get_pname(df['pdgID'].values)
    penum = {pname: i for i, pname in enumerate(df['pname'].unique())}
    df.loc[:, 'pname_id'] = df['pname'].map(penum)

    ax = axs.flat[0]
    ax.hist(df['pname_id'].values, bins=np.arange(-4, len(penum)*2 + 4), align='left', ec='k')
    ax.set_ylabel('Particle Counts', fontsize=24)
    ax.tick_params(labelsize=22)
    ax.set_axisbelow(True)
    ax.grid(linestyle=':', which='both', axis='y')
    ax.xaxis.set_major_locator(ticker.FixedLocator(list(penum.values())))
    ax.set_xticklabels(list(penum), rotation=30)

    # calculate kinematics
    get_4vecs = np.vectorize(lambda x, y, z, m: ROOT.Math.PxPyPzMVector(x, y, z, m))
    fourvecs = get_4vecs(*df[['psx', 'psy', 'psz', 'mass']].values.T)

    df.loc[:, 'p'] = [v.P() for v in fourvecs]
    df.loc[:, 'eta'] = [v.Eta() for v in fourvecs]
    df.loc[:, 'pt'] = [v.Pt() for v in fourvecs]

    # column name, x label, y label, bins
    plots = [
        ('p', r'$P$ (GeV)', 'Counts', 50),
        ('eta', r'$\eta$', 'Counts', 50),
        ('pt', r'$P_T$ (GeV)', 'Counts', 50),
    ]
    for ax, (col, xlabel, ylabel, bins) in zip(axs.flat[1:], plots):
        ax.hist(df[col].values, bins=bins, align='left', ec='k')
        ax.tick_params(labelsize=22, direction='in', which='both')
        ax.grid(linestyle=':', which='both')
        ax.set_axisbelow(True)
        ax.set_xlabel(xlabel, fontsize=24)
        ax.set_ylabel(ylabel, fontsize=24)

    fig.text(0.5, 0.95, 'Thrown Particles', ha='center', fontsize=24)
    fig.savefig(save)
    plt.close(fig)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('sim_file', help='Path to simulation output file.')
    parser.add_argument('rec_file', help='Path to reconstruction output file.')
    parser.add_argument('-o', dest='outdir', default='.', help='Output directory.')
    parser.add_argument('-m', '--macros', dest='macros', default='rootlogon.C',
            help='Macro files to be loaded by root, separated by \",\".')
    parser.add_argument('--mc-branch', dest='mc', default='mcparticles',
            help='Branch name of MC particles truth info.')
    parser.add_argument('--collections', dest='coll', default='',
            help='Collection name of clusters to plot')
    args = parser.parse_args()

    # multi-threading for RDataFrame
    nthreads = os.cpu_count()//2
    ROOT.ROOT.EnableImplicitMT(nthreads)
    print('CPU available {}, using {}'.format(os.cpu_count(), nthreads))

    # declare some functions from c++ script, needed for data frame processing
    # script_dir = os.path.dirname(os.path.realpath(__file__))
    # fcode = open(os.path.join(script_dir, 'barrel_clusters.cxx')).read()
    # ROOT.gInterpreter.Declare(fcode)

    os.makedirs(args.outdir, exist_ok=True)
    # load macros (add libraries/headers root cannot automatically found here)
    load_root_macros(args.macros)

    rdf_sim = ROOT.RDataFrame('events', args.sim_file)
    thrown_particles_figure(rdf_sim, save=os.path.join(args.outdir, 'thrown_particles.png'), mcbranch=args.mc)

    rdf_rec = ROOT.RDataFrame('events', args.rec_file)
    # cluster collections
    colls = [c.strip() for c in args.coll.split(',')]
    # a function to extract eta
    ROOT.gInterpreter.Declare('''
        template<typename T>
        ROOT::VecOps::RVec<T> eta(const ROOT::VecOps::RVec<T> &theta) {
            ROOT::VecOps::RVec<double> result;
            for (size_t i = 0; i < theta.size(); ++ i) { result.push_back(-std::log(std::tan(theta[i]/2.))); }
            return result;
        }
    ''')

    # branch name, x label, x bins
    plots = [
        ('nhits', 'Number of Hits', 50),
        ('energy', 'Energy (GeV)', 50),
        ('eta', '$\eta$', 50),
    ]
    for coll in colls:
        df = flatten_collection(rdf_rec, coll)
        if not df.shape[0]:
            print('{} do not have valid entries, skip it'.format(coll))
            continue
        df.rename(columns={c: c.replace(coll + '.', '') for c in df.columns}, inplace=True)
        # calculate eta
        if 'eta' not in df.columns:
            df.loc[:, 'eta'] = -np.log(np.tan(df['polar.theta'].values/2.))
        # print(df[['eta', 'polar.theta', 'position.x', 'position.y', 'position.z']])
        fig, axs = plt.subplots(2, 2, figsize=(12, 8), dpi=160)
        ncl = df.groupby('event')['clusterID'].nunique().values
        axs[0][0].hist(ncl, weights=np.repeat(1./float(ncl.shape[0]), ncl.shape[0]),
                bins=np.arange(0, 10), align='mid', ec='k')
        axs[0][0].set_xlabel('Number of Clusters', fontsize=16)

        for ax, (branch, xlabel, bins) in zip(axs.flat[1:], plots):
            ax.hist(df[branch].values, weights=np.repeat(1./float(df.shape[0]), df.shape[0]),
                    bins=bins, align='mid', ec='k')
            ax.set_xlabel(xlabel, fontsize=16)

        # universal axis settings
        for ax in axs.flat:
            ax.tick_params(labelsize=16, direction='in')
            ax.set_axisbelow(True)
            ax.grid(linestyle=':')
        fig.text(0.06, 0.5, 'Normalized Counts', rotation=90, va='center', fontsize=18)
        fig.savefig(os.path.join(args.outdir, '{}.png'.format(coll)))

