'''
    A simple analysis script to extract some basic info of clusters
    Chao Peng (ANL), 07/08/2021
'''
import os
import ROOT
import pandas as pd
import numpy as np
import argparse
from matplotlib import pyplot as plt
import matplotlib.ticker as ticker


# load root macros, input is an argument string
def load_root_macros(arg_macros):
    for path in arg_macros.split(','):
        path = path.strip()
        if os.path.exists(path):
            gROOT.Macro(path)
            print('Loading root macro: \'{}\' loaded.'.format(path))
        else:
            print('Loading root macros: \'{}\' does not exist, skip loading it.'.format(path))


# read from RDataFrame and flatten a given collection, return pandas dataframe
def flatten_collection(rdf, collection, cols=None):
    if not cols:
        cols = [str(c) for c in rdf.GetColumnNames() if str(c).startswith('{}.'.format(collection))]
    else:
        cols = ['{}.{}'.format(collection, c) for c in cols]
    if not cols:
        print('cannot find any branch under collection {}'.format(collection))
        return pd.DataFrame()

    data = rdf.AsNumpy(cols)
    # flatten the data, add an event id to identify clusters from different events
    evns = []
    for i, vec in enumerate(data[cols[0]]):
        evns += [i]*vec.size()
    for n, vals in data.items():
        # make sure ints are not converted to floats
        typename = vals[0].__class__.__name__.lower()
        dtype = np.int64 if 'int' in typename or 'long' in typename else np.float64
        # type safe creation
        data[n] = np.asarray([v for vec in vals for v in vec], dtype=dtype)
    # build data frame
    dfp = pd.DataFrame({c: pd.Series(v) for c, v in data.items()})
    dfp.loc[:, 'event'] = evns
    return dfp


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('rec_file', help='Path to reconstruction output file.')
    parser.add_argument('-o', dest='outdir', default='.', help='Output directory.')
    parser.add_argument('-m', '--macros', dest='macros', default='rootlogon.C',
            help='Macro files to be loaded by root, separated by \",\".')
    parser.add_argument('--collections', dest='coll', default='',
            help='Collection names for clusters, separated by \",\"')
    args = parser.parse_args()

    # multi-threading for RDataFrame
    nthreads = os.cpu_count()//2
    ROOT.ROOT.EnableImplicitMT(nthreads)
    print('CPU available {}, using {}'.format(os.cpu_count(), nthreads))

    os.makedirs(args.outdir, exist_ok=True)
    # load macros (add libraries/headers root cannot automatically found here)
    load_root_macros(args.macros)

    rdf_rec = ROOT.RDataFrame('events', args.rec_file)

    if not args.coll:
        print('No collection names provided, no plots generated.')
        exit(-1)

    # a function to extract eta
    ROOT.gInterpreter.Declare('''
        template<typename T>
        ROOT::VecOps::RVec<T> eta(const ROOT::VecOps::RVec<T> &theta) {
            ROOT::VecOps::RVec<double> result;
            for (size_t i = 0; i < theta.size(); ++ i) { result.push_back(-std::log(std::tan(theta[i]/2.))); }
            return result;
        }
    ''')

    # branch name, x label, x bins
    plots = [
        ('nhits', 'Number of Hits', 50),
        ('energy', 'Energy (GeV)', 50),
        ('eta', '$\eta$', 50),
    ]
    for coll in [c.strip() for c in args.coll.split(',')]:
        df = flatten_collection(rdf_rec, coll)
        # calculate eta
        df.loc[:, '{}.eta'.format(coll)] = -np.log(np.tan(df['{}.polar.theta'.format(coll)].values/2.))
        fig, axs = plt.subplots(2, 2, figsize=(12, 8), dpi=160)
        ncl = df.groupby('event')['{}.clusterID'.format(coll)].nunique().values
        axs[0][0].hist(ncl, weights=np.repeat(1./float(ncl.shape[0]), ncl.shape[0]),
                bins=np.arange(0, 10), align='mid', ec='k')
        axs[0][0].set_xlabel('Number of Clusters', fontsize=16)

        for ax, (branch, xlabel, bins) in zip(axs.flat[1:], plots):
            ax.hist(df['{}.{}'.format(coll, branch)].values, weights=np.repeat(1./float(df.shape[0]), df.shape[0]),
                    bins=bins, align='mid', ec='k')
            ax.set_xlabel(xlabel, fontsize=16)

        # universal axis settings
        for ax in axs.flat:
            ax.tick_params(labelsize=16, direction='in')
            ax.set_axisbelow(True)
            ax.grid(linestyle=':')
        fig.text(0.06, 0.5, 'Normalized Counts', rotation=90, va='center', fontsize=18)
        fig.savefig(os.path.join(args.outdir, '{}.png'.format(coll)))


