import ROOT
import os
import numpy as np
import pandas as pd
import argparse
import sys


# read hits data from root file
def get_ml_data(path, evnums=None, shape=(20, 20, 3), branch='EcalBarrelHitsML'):
    f = ROOT.TFile(path)
    events = f.events
    if evnums is None:
        evnums = np.arange(events.GetEntries())
    elif isinstance(evnums, int):
        evnums = [evnums]

    dbuf = np.zeros(shape=np.hstack([len(evnums), shape]))
    idb = 0
    for iev in evnums:
        if iev >= events.GetEntries():
            print('Error: event {:d} is out of range (0 - {:d})'.format(iev, events.GetEntries() - 1))
            continue

        events.GetEntry(iev)
        for hit in getattr(events, branch):
            dbuf[iev][hit.layerID][hit.hitID] = (hit.edep, hit.eta, hit.polar.phi)

    return dbuf


# execute this script
# read mc particles from root file
def get_mcp_simple(path, evnums=None, branch='mcparticles2'):
    f = ROOT.TFile(path)
    events = f.events
    if evnums is None:
        evnums = np.arange(events.GetEntries())
    elif isinstance(evnums, int):
        evnums = [evnums]

    dbuf = np.zeros(shape=(len(evnums), 6))
    idb = 0
    for iev in evnums:
        if iev >= events.GetEntries():
            print('Error: event {:d} is out of range (0 - {:d})'.format(iev, events.GetEntries() - 1))
            continue

        events.GetEntry(iev)
        # extract full mc particle data
        part = getattr(events, branch)[2]
        dbuf[idb] = (iev, part.psx, part.psy, part.psz, part.pdgID, part.status)
        idb += 1
    return pd.DataFrame(data=dbuf[:idb], columns=['event', 'px', 'py', 'pz', 'pid', 'status'])


# load root macros, input is an argument string
def load_root_macros(arg_macros):
    for path in arg_macros.split(','):
        path = path.strip()
        if os.path.exists(path):
            ROOT.gROOT.Macro(path)
        else:
            print('\"{}\" does not exist, skip loading it.'.format(path))


if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Visualize the cluster (layer-wise) from analysis')
    parser.add_argument('file', type=str, help='path to root file')
    parser.add_argument('-o', type=str, default='.', dest='outdir', help='output directory')
    # parser.add_argument('--compact', type=str, default='', dest='compact', help='compact file')
    parser.add_argument('-m', '--macros', type=str, default='rootlogon.C', dest='macros',
                         help='root macros to load (accept multiple paths separated by \",\")')
    parser.add_argument('--topo-size', type=float, default=2.0, dest='topo_size',
                        help='bin size for projection plot (mrad)')
    parser.add_argument('--topo-range', type=float, default=200.0, dest='topo_range',
                        help='half range for projection plot (mrad)')
    parser.add_argument('--shape', type=str, default="20, 20, 3", dest='shape',
                        help='shape of data (3D), default (20, 20, 3)')
    args = parser.parse_args()


    os.makedirs(args.outdir, exist_ok=True)
    load_root_macros(args.macros)
    data = get_ml_data(args.file, shape=[int(i.strip()) for i in args.shape.split(',')])
    dfmcp = get_mcp_simple(args.file, branch='mcparticles2')
    # dfmcp = dfmcp[dfmcp['status'] == 24578]
    dfmcp.loc[:, 'p'] = np.linalg.norm(dfmcp[['px', 'py', 'pz']].values, axis=1)
    dfmcp['phi'] = np.arctan2(dfmcp['py'].values, dfmcp['px'].values)
    dfmcp['theta'] = np.arccos(dfmcp['pz'].values/dfmcp['p'].values)
    dfmcp['eta'] = -np.log(np.tan(dfmcp['theta'].values/2.))

    tag = np.zeros(shape=(data.shape[0], 7))
    trg = args.topo_range/1000.

    for iev in np.arange(data.shape[0]):
        ev = data[iev].transpose(2,0,1).reshape(3,-1)
        truth = dfmcp[dfmcp['event'] == iev]
        if not truth.shape[0]:
            continue
        mask = ev[0] > 0.
        if not sum(mask):
            continue
        eta = ev[1][ev[0] > 0.]
        phi = ev[2][ev[0] > 0.]
        # normalize
        # print(ev.T.reshape(20, 20, 3) == data[iev])
        ev[0] /= 10.
        # ev[1][mask] = (ev[1][mask] - eta.mean() + trg)/(2.*trg)
        # ev[2][mask] = (ev[2][mask] - phi.mean() + trg)/(2.*trg)
        ev[1][mask] = (ev[1][mask] - truth['eta'].values + trg)/(2.*trg)
        ev[2][mask] = (ev[2][mask] - truth['phi'].values + trg)/(2.*trg)
        # ev[1][mask] = (ev[1][mask] + 1.0)/2.
        # ev[2][mask] = (ev[2][mask] + np.pi)/2./np.pi
        out_range = (ev[1] > 1.) | (ev[1] < 0.) | (ev[2] > 1.) | (ev[2] < 0.)
        ev.T[out_range] = (0., 0., 0.)
        test = data[iev].transpose(2,0,1).reshape(3,-1)
        tag[iev] = np.hstack([truth[['pid', 'eta', 'phi', 'p']].values[0], (eta.mean(), phi.mean(), trg)])
        # print(ev[1])
        # print((eta.shape, phi.shape), (eta.mean(), phi.mean()), truth[['eta', 'phi']].values)

    # momentum
    mask = tag.T[3] > 0.
    print('training data shape: {}'.format(data[mask].shape))
    print('tagging data shape: {}'.format(tag[mask].shape))
    np.save(os.path.join(args.outdir, 'data.npy'), data[mask])
    np.save(os.path.join(args.outdir, 'tag.npy'), tag[mask])

