#! /usr/local/bin/python3
'''
    A python script to facilitate the ML training benchmark for imaging Calorimeter:
    simulation -> reconstruction -> prepare tagging data for tensorflow
    Author: Chao Peng (ANL)
    Date: 06/20/2021
'''
import os
import sys
import subprocess
import argparse


default_compact = os.path.join(os.environ.get('JUGGLER_DETECTOR_PATH',  os.environ.get('DETECTOR_PATH', '')),
        '{}.xml'.format(os.environ.get('JUGGLER_DETECTOR', '')))
parser = argparse.ArgumentParser()
parser.add_argument('-n', '--numberOfEvents', dest='nev', type=int, default=100, help='Number of events to process.')
parser.add_argument('-t', '--nametag', type=str, default='IMCAL_ML', help='Name tag for output files.')
parser.add_argument('--seed', type=int, default=-1, help='Random seed to scripts.')
parser.add_argument('--process', type=str, default='sim, rec, tag', help='Processes to be executed (sim, rec, tag).')
parser.add_argument('--numberOfLayers', dest='nlayer', type=int, default=20, help='number of layers in ML data.')
parser.add_argument('--numberOfHits', dest='nhit', type=int, default=20, help='number of hits in ML data.')
parser.add_argument('--particles', type=str, default='electron', help='Partcile names, separated by \",\".')
parser.add_argument('--pmin', type=float, default=0.5, help='Minimum momentum of particles.')
parser.add_argument('--pmax', type=float, default=10, help='Maximum momentum of particles.')
parser.add_argument('--compact', type=str, default=default_compact, help='Path to detector compact file.')

args = parser.parse_args()
kwargs = vars(args)

for mdir in ['gen_data', 'sim_data', 'rec_data', 'tag_data']:
    os.makedirs(mdir, exist_ok=True)

gen_file = os.path.join('gen_data', '{nametag}_{pmin}_{pmax}.hepmc'.format(**kwargs))
sim_file = os.path.join('sim_data', '{nametag}_{pmin}_{pmax}.root'.format(**kwargs))
rec_file = os.path.join('rec_data', '{nametag}_{pmin}_{pmax}.root'.format(**kwargs))
tag_dir = os.path.join('tag_data', '{nametag}_{pmin}_{pmax}'.format(**kwargs))
os.makedirs(tag_dir, exist_ok=True)

procs = [p.strip() for p in args.process.split(',')]
sdir = os.path.dirname(os.path.realpath(__file__))

if 'sim' in procs:
    # generate particles
    gen_cmd = ['python', os.path.join(sdir, 'scripts', 'gen_particles.py'), gen_file,
            '-n', '{}'.format(args.nev),
            '-s', '{}'.format(args.seed),
            '--angmin', '50', '--angmax', '130',
            '--pmin', '{}'.format(args.pmin), '--pmax', '{}'.format(args.pmax),
            '--particles', args.particles]
    subprocess.run(gen_cmd)
    # simulation
    sim_cmd = ['npsim',
            '--part.minimalKineticEnergy', '1*TeV',
            '--numberOfEvents', '{}'.format(args.nev),
            '--runType', 'batch',
            '--inputFiles', gen_file,
            '--outputFile', sim_file,
            '--compact', args.compact,
            '-v', 'WARNING']
    if args.seed > 0:
        sim_cmd += ['--random.seed', args.seed]
    return_code = subprocess.run(sim_cmd).returncode
    print(return_code)
    if return_code is not None and return_code < 0:
        print("ERROR running simulation!")
        exit(return_code)
    subprocess.run(['rootls', '-t', sim_file])


if 'rec' in procs:
    # export to environment variables (used to pass arguments to the option file)
    os.environ['JUGGLER_SIM_FILE'] = sim_file
    os.environ['JUGGLER_REC_FILE'] = rec_file
    os.environ['JUGGLER_COMPACT_PATH'] = args.compact
    os.environ['JUGGLER_N_EVENTS'] = '{}'.format(args.nev)
    os.environ['IMCAL_ML_N_LAYERS'] = '{}'.format(args.nlayer)
    os.environ['IMCAL_ML_N_HITS'] = '{}'.format(args.nhit)

    juggler_xenv = os.path.join(os.environ.get('JUGGLER_INTALL_PREFIX', '../local'), 'Juggler.xenv')
    rec_cmd = ['xenv', '-x', juggler_xenv, 'gaudirun.py', os.path.join(sdir, 'options', 'imaging_ml_data.py')]
    return_code = subprocess.run(rec_cmd).returncode
    print(return_code)
    if return_code is not None and return_code < 0:
        print("ERROR running juggler (reconstruction)!")
        exit(return_code)
    process = subprocess.run(['rootls', '-t', rec_file])


if 'tag' in procs:
    tag_cmd = ['python', os.path.join(sdir, 'scripts', 'prepare_tf_dataset.py'), rec_file,
            '-o', tag_dir,
            '--shape', '{nlayer},{nhit},3'.format(**kwargs)]
    return_code = subprocess.run(tag_cmd).returncode
    print(return_code)
    if return_code is not None and return_code < 0:
        print("ERROR running ML data tagging!")
        exit(return_code)

